import json
import os
from subprocess import call, check_output


def getServices(includeOnlyBuildable=False):
    data = check_output(['docker', 'compose', 'config', '--format', 'json'])
    config = json.loads(data)
    services = config['services']
    if includeOnlyBuildable:
        services = {name: values for name, values in services.items() if 'build' in values.keys()}
    return services


def getServicesRunning():
    data = check_output(['docker', 'compose', 'ps', '--format', 'json'])
    try:
        lines = data.decode().splitlines()
        if len(lines) > 1:
            container_list = []
            for l in lines:
                #print(f'l: {l}')
                container_list.append(json.loads(l.strip()))
        else:
            container_list = json.loads(data)
    except Exception as e:
        print(e)
        print(data.decode())
    if isinstance(container_list, dict):
        container_names = [container_list['Name']]
    else:
        container_names = [s['Name'] for s in container_list]
    return container_names


def getImages():
    data = check_output(['docker', 'compose', 'images', '--format', 'json'])
    image_list = json.loads(data)
    images = [
        {'ID': image['ID'],
         'Name': image['Repository'],
         'ContainerName': image['ContainerName'],
         'Tag': image['Tag']}
        for image in image_list
    ]
    return images


def getStats():
    containers = getServicesRunning()
    data = check_output(['docker', 'stats', '--format', 'json', '--no-stream'] + containers, text=True).strip()
    data_lines = data.split('\n')
    stats = [json.loads(line) for line in data_lines]
    return stats


def runCompose(args, **kwargs):
    command = ['docker', 'compose']
    if os.environ.get("PORTMGR_IN_SCRIPT", "").lower() == "true":
        command += ["--ansi", "never"]
    command += list(args)
    return call(command, **kwargs)

def runBuildx(args, **kwargs):
    command = ['docker', 'buildx']
    command += list(args)
    return call(command, **kwargs)



#!/usr/bin/env python3
from __future__ import unicode_literals

import sys

if __package__ is None and not hasattr(sys, 'frozen'):
    # direct call of __main__.py
    import os.path
    path = os.path.realpath(os.path.abspath(__file__))
    sys.path.insert(0, os.path.dirname(os.path.dirname(path)))

import portmgr

if __name__ == '__main__':
    portmgr.main()



from portmgr import command_list
import subprocess

from portmgr.wrapper import getImages


def func(action):
    directory = action['directory']
    relative = action['relative']

    images = getImages()

    res = 0

    for image in images:
        image_name = image["Name"]
        print(f'Scanning {image_name} of {image["ContainerName"]}')
        scan_res = subprocess.run(['trivy', '-q', 'image', '-s', 'CRITICAL', '--exit-code', '1', image_name], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
        if scan_res.returncode != 0:
            print('Found vulnerabilites:')
            print(scan_res.stdout)
            res = scan_res.returncode

      #print("Name: %s" % container.name)
      #names.append(container.name)
      #config_str = subprocess.check_output(["docker", "inspect", "-f", '{{json .}}', container.id], stdout=subprocess.PIPE)
      #json.loads(config_str)
      #print("IP: %s" % ip)
      #print(container.inspect)


    #ID = subprocess.run(["docker-compose", "ps", '-q'], stdout=subprocess.PIPE).stdout

    #if res != 0:
    #    print("Error listing containers for " + relative + "!\n")

    return res

command_list['v'] = {
    'hlp': 'Scan container images for vulnerabilities',
    'ord': 'nrm',
    'fnc': func
}



from portmgr import command_list, bcolors, runCompose

def func(action):
    directory = action['directory']
    relative = action['relative']

    res = runCompose(["stop"])
    # p = subprocess.Popen(["docker-compose", "stop"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

     #out, err = p.communicate()

     #if out != "":
     #    print(out.decode("UTF-8"))

    if res != 0:
        print("Error stopping " + relative + "!")
    
    # print(bcolors.FAIL + err.decode("UTF-8") + bcolors.ENDC)

    return 0

command_list['s'] = {
    'hlp': 'Stop container',
    'ord': 'rev',
    'fnc': func
}
